#pragma once
///////////////////////////////////////////////////////////////////////////////
// File: HTTPSocketServer.h
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2016 JetByte Limited.
//
// This software is provided "as is" without a warranty of any kind. All
// express or implied conditions, representations and warranties, including
// any implied warranty of merchantability, fitness for a particular purpose
// or non-infringement, are hereby excluded. JetByte Limited and its licensors
// shall not be liable for any damages suffered by licensee as a result of
// using the software. In no event will JetByte Limited be liable for any
// lost revenue, profit or data, or for direct, indirect, special,
// consequential, incidental or punitive damages, however caused and regardless
// of the theory of liability, arising out of the use of or inability to use
// software, even if JetByte Limited has been advised of the possibility of
// such damages.
//
///////////////////////////////////////////////////////////////////////////////

#include "JetByteTools/SocketTools/StreamSocketServerEx.h"
#include "JetByteTools/SocketTools/StreamSocketServerExCallback.h"
#include "JetByteTools/SocketTools/FlowControlStreamSocketConnectionFilter.h"
#include "JetByteTools/SocketTools/IMonitorStreamSocketFlowControl.h"

#include "JetByteTools/CoreTools/tstring.h"

///////////////////////////////////////////////////////////////////////////////
// Classes defined in other files...
///////////////////////////////////////////////////////////////////////////////

namespace JetByteTools
{
   namespace HTTP
   {
      class IManageResources;
   }
}

///////////////////////////////////////////////////////////////////////////////
// CHTTPSocketServer
///////////////////////////////////////////////////////////////////////////////

class CHTTPSocketServer :
   public JetByteTools::Socket::CFilteringStreamSocketServerEx,
   private JetByteTools::Socket::CStreamSocketServerExCallback,
   private JetByteTools::Socket::CFlowControlStreamSocketConnectionFilter::IProcessBufferList,
   private JetByteTools::Socket::IMonitorStreamSocketFlowControl
{
   public :

      CHTTPSocketServer(
         JetByteTools::HTTP::IManageResources &resourceManager,
         const JetByteTools::Socket::CFlowControlStreamSocketConnectionFilter::BufferCount maxBuffers,
         const JetByteTools::Socket::CFlowControlStreamSocketConnectionFilter::PendingWriteCount maxPendingWrites,
         const JetByteTools::Socket::CFlowControlStreamSocketConnectionFilter::Percentage notificationPercentageStepSize,
         const JetByteTools::Socket::SocketBufferSize recvBufferSize,
         const JetByteTools::Socket::SocketBufferSize sendBufferSize,
         const JetByteTools::Socket::IFullAddress &address,
         const JetByteTools::Socket::ListenBacklog listenBacklog,
         JetByteTools::IO::IIOPool &pool,
         JetByteTools::Socket::IAllocateSequencedStreamSockets &socketAllocator,
         JetByteTools::IO::IAllocateBuffers &bufferAllocator);

      CHTTPSocketServer(
         const CHTTPSocketServer &rhs) = delete;

      ~CHTTPSocketServer();

      CHTTPSocketServer &operator=(
         const CHTTPSocketServer &rhs) = delete;

   private :

      // Implement CSocketServer

      void OnConnectionEstablished(
         JetByteTools::Socket::IStreamSocket &socket,
         const JetByteTools::Socket::IAddress &address) override;

      void OnSocketReleased(
         JetByteTools::Core::IIndexedOpaqueUserData &userData) override;

      void OnReadCompleted(
         JetByteTools::Socket::IStreamSocket &socket,
         JetByteTools::IO::CSmartBuffer &buffer) override;

      void OnConnectionClientClose(
          JetByteTools::Socket::IStreamSocket &socket) override;

      // Implement CFlowControlStreamSocketConnectionFilter::IProcessBufferList

      bool OnListFull(
         JetByteTools::Socket::IStreamSocket &socket,
         const JetByteTools::Socket::IMonitorStreamSocketFlowControl::ConnectionId id,
         JetByteTools::Socket::IMonitorStreamSocketFlowControl &monitor,
         const JetByteTools::IO::IBuffer &newBuffer,
         JetByteTools::IO::IBufferChain &buffers,
         JetByteTools::IO::IBufferChain &discardedBuffers) override;

      void OnListStatus(
         JetByteTools::Socket::IStreamSocket &socket,
         const DWORD percentageUsed) override;

      void OnListEmpty(
         JetByteTools::Socket::IStreamSocket &socket) override;

      // Implement IMonitorStreamSocketFlowControl

      ConnectionId OnConnectionEstablished(
         JetByteTools::Socket::IStreamSocket &socket) override;

      void OnWritePending(
         const ConnectionId id) override;

      void OnWriteComplete(
         const ConnectionId id) override;

      bool OnWriteCompletionError(
         const ConnectionId id,
         JetByteTools::Socket::IStreamSocket &socket,
         JetByteTools::IO::IBuffer &buffer,
         const DWORD lastError) override;

      void OnBufferDelayed(
         const ConnectionId id,
         JetByteTools::IO::IBuffer &buffer) override;

      void OnBufferSent(
         const ConnectionId id,
         JetByteTools::IO::IBuffer &buffer) override;

      void OnBufferDiscarded(
         const ConnectionId id,
         JetByteTools::IO::IBuffer &buffer,
         const bool wasDelayed) override;

      void OnConnectionComplete(
         const ConnectionId id) override;

      void OnConnectionTerminated(
         const ConnectionId id) override;

      JetByteTools::Socket::CFlowControlStreamSocketConnectionFilter m_flowControlFilter;

      JetByteTools::HTTP::IManageResources &m_resourceManager;

      const JetByteTools::Core::IIndexedOpaqueUserData::UserDataIndex m_connectionIndex;
};

///////////////////////////////////////////////////////////////////////////////
// End of file: HTTPSocketServer.h
///////////////////////////////////////////////////////////////////////////////
