#pragma once
///////////////////////////////////////////////////////////////////////////////
// File: CommandLine.h
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2005 JetByte Limited.
//
// This software is provided "as is" without a warranty of any kind. All
// express or implied conditions, representations and warranties, including
// any implied warranty of merchantability, fitness for a particular purpose
// or non-infringement, are hereby excluded. JetByte Limited and its licensors
// shall not be liable for any damages suffered by licensee as a result of
// using the software. In no event will JetByte Limited be liable for any
// lost revenue, profit or data, or for direct, indirect, special,
// consequential, incidental or punitive damages, however caused and regardless
// of the theory of liability, arising out of the use of or inability to use
// software, even if JetByte Limited has been advised of the possibility of
// such damages.
//
///////////////////////////////////////////////////////////////////////////////

#include "JetByteTools/CoreTools/Types.h"
#include "JetByteTools/CoreTools/CommandLine.h"

#include "JetByteTools/SocketTools/ConsecutiveSocketOperations.h"
#include "JetByteTools/SocketTools/PerSocketBufferPoolingLimit.h"

///////////////////////////////////////////////////////////////////////////////
// CCommandLine
///////////////////////////////////////////////////////////////////////////////

class CCommandLine
{
   public :

      typedef unsigned short Count;

      CCommandLine();

      CCommandLine(
         const CCommandLine &rhs) = delete;

      CCommandLine &operator=(
         const CCommandLine &rhs) = delete;

      bool Parse(
         DWORD argc,
         const char **argv);

      JetByteTools::Core::_tstring Server() const;

      unsigned short Port() const;

      Count Connections() const;

      Count ConnectionBatchSize() const;

      JetByteTools::Milliseconds ConnectionBatchPause() const;

      Count Messages() const;

      Count MessageSize() const;

      JetByteTools::Milliseconds MessageRate() const;

      bool IsInteractive() const;

      bool SendAfterRecv() const;

      bool SyncConnect() const;

      bool HoldConnections() const;

      bool PreAllocate() const;

      bool Fragments() const;

      bool MultipleMessages() const;

      JetByteTools::Milliseconds MessageFlowTimeout() const;

      bool DisplayConnectionBatching() const;

      JetByteTools::Milliseconds CheckEvery() const;

      JetByteTools::Milliseconds ReadTimeout() const;

      Count NumberOfIOThreads() const;

      bool UseSSL() const;

      bool VerifyServerCert() const;

      JetByteTools::Core::_tstring CertificatePath() const;

      JetByteTools::Core::_tstring Password() const;

      JetByteTools::Socket::PerSocketBufferPoolingLimit MaxBuffersPooledPerSocket() const;

      JetByteTools::Socket::ConsecutiveSocketOperations MaxConsecutiveOperationsOnOneSocket() const;

      void ShowHelp(
         const JetByteTools::Core::_tstring &message);

   private :

      static Count StringToCount(
         const JetByteTools::Core::_tstring &stringValue);

      JetByteTools::Core::CCommandLine m_parser;
};

///////////////////////////////////////////////////////////////////////////////
// End of file: CommandLine.h
///////////////////////////////////////////////////////////////////////////////
