///////////////////////////////////////////////////////////////////////////////
// File: TestMessage.cpp
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2005 JetByte Limited.
//
// This software is provided "as is" without a warranty of any kind. All
// express or implied conditions, representations and warranties, including
// any implied warranty of merchantability, fitness for a particular purpose
// or non-infringement, are hereby excluded. JetByte Limited and its licensors
// shall not be liable for any damages suffered by licensee as a result of
// using the software. In no event will JetByte Limited be liable for any
// lost revenue, profit or data, or for direct, indirect, special,
// consequential, incidental or punitive damages, however caused and regardless
// of the theory of liability, arising out of the use of or inability to use
// software, even if JetByte Limited has been advised of the possibility of
// such damages.
//
///////////////////////////////////////////////////////////////////////////////

#include "JetByteTools/Admin/Admin.h"

#include "TestMessage.h"

#include "JetByteTools/CoreTools/Exception.h"
#include "JetByteTools/CoreTools/FileUtils.h"
#include "JetByteTools/CoreTools/StringConverter.h"

#include "JetByteTools/HTTPTools/HTTPResponse.h"

#pragma hdrstop

///////////////////////////////////////////////////////////////////////////////
// Using directives
///////////////////////////////////////////////////////////////////////////////

using JetByteTools::Core::_tstring;
using JetByteTools::Core::CException;
using JetByteTools::Core::CStringConverter;

using std::string;

///////////////////////////////////////////////////////////////////////////////
// Static helper functions
///////////////////////////////////////////////////////////////////////////////

static string BuildRequest(
   const _tstring &url,
   const _tstring &host);

///////////////////////////////////////////////////////////////////////////////
// CTestMessage
///////////////////////////////////////////////////////////////////////////////

CTestMessage::CTestMessage(
   const _tstring &url,
   const _tstring &host)
   :  m_url(url),
      m_request(BuildRequest(url, host)),
      m_sendOffset(0),
      m_sendIndex(0),
      m_recvIndex(0)
{
}

// Implement ITestMessage

void CTestMessage::ExpectRepeats(
   const size_t numberOfRepeats)
{
   m_times.resize(numberOfRepeats);
}

_tstring CTestMessage::GetIdentifier() const
{
   return _T("HTTP Get: ") + m_url;
}

const CTestMessage::Times &CTestMessage::GetTimes() const
{
   return m_times;
}

const BYTE *CTestMessage::GetBytes() const
{
   return reinterpret_cast<const BYTE *>(m_request.c_str()) + m_sendOffset;
}

CTestMessage::DataLength CTestMessage::GetSize() const
{
   return static_cast<DataLength>(m_request.length());
}

bool CTestMessage::ValidateResponse(
   const BYTE * const pData,
   const DataLength dataLength,
   DataLength &dataUsed) const
{
   const auto ticks = ::GetTickCount();

   m_times[m_recvIndex].recvTime = ticks;

   (void)pData;
   (void)dataLength;
   (void)dataUsed;

   return false;
}

CTestMessage::DataLength CTestMessage::GetRecvOffset() const
{
   return 0;
}

CTestMessage::DataLength CTestMessage::GetResponseSize() const
{
   return 0;
}

bool CTestMessage::BytesSent(
   const DataLength bytesSent)
{
   auto allSent = false;

   m_sendOffset += bytesSent;

   if (m_sendOffset == m_request.length())
   {
      m_sendOffset = 0;

      const auto ticks = ::GetTickCount();

      m_times[m_sendIndex].sentTime = ticks;
      m_times[m_sendIndex].recvTime = ticks;

      m_sendIndex++;

      allSent = true;
   }
   else if (m_sendOffset > m_request.length())
   {
      throw CException(_T("CTestMessage::BytesSent()"), _T("More bytes sent than there are in the message"));
   }

   return allSent;
}

void CTestMessage::ResetMessage()
{
   m_recvIndex++;
}

///////////////////////////////////////////////////////////////////////////////
// Static helper functions
///////////////////////////////////////////////////////////////////////////////

static string BuildRequest(
   const _tstring &url,
   const _tstring &host)
{
   auto request = "GET " + CStringConverter::TtoA(url) + " HTTP/1.1\r\n"
      "Host: " + CStringConverter::TtoA(host) + "\r\n"
      "Connection: keep-alive\r\n"
      "\r\n";

   return request;
}

///////////////////////////////////////////////////////////////////////////////
// End of file: TestMessage.cpp
///////////////////////////////////////////////////////////////////////////////
