///////////////////////////////////////////////////////////////////////////////
// File: IOPool.cpp
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2004 JetByte Limited.
//
// This software is provided "as is" without a warranty of any kind. All
// express or implied conditions, representations and warranties, including
// any implied warranty of merchantability, fitness for a particular purpose
// or non-infringement, are hereby excluded. JetByte Limited and its licensors
// shall not be liable for any damages suffered by licensee as a result of
// using the software. In no event will JetByte Limited be liable for any
// lost revenue, profit or data, or for direct, indirect, special,
// consequential, incidental or punitive damages, however caused and regardless
// of the theory of liability, arising out of the use of or inability to use
// software, even if JetByte Limited has been advised of the possibility of
// such damages.
//
///////////////////////////////////////////////////////////////////////////////

#include "JetByteTools/Admin/Admin.h"

#include "IOPool.h"

#include "JetByteTools/CoreTools/StringUtils.h"
#include "JetByteTools/CoreTools/DebugTrace.h"
#include "JetByteTools/CoreTools/ToString.h"

#pragma hdrstop

///////////////////////////////////////////////////////////////////////////////
// Using directives
///////////////////////////////////////////////////////////////////////////////

using JetByteTools::Core::Output;
using JetByteTools::Core::OutputEx;
using JetByteTools::Core::_tstring;
using JetByteTools::Core::ToString;
using JetByteTools::Core::BoolAsString;

///////////////////////////////////////////////////////////////////////////////
// CIOPool
///////////////////////////////////////////////////////////////////////////////

CIOPool::CIOPool(
   const ThreadCount numThreads,
   const bool log)
   :  JetByteTools::IO::CIOPool(*this, numThreads),
      m_log(log)
{

}

CIOPool::~CIOPool()
{
   WaitForShutdownToComplete();
}

void CIOPool::OnThreadPoolError(
   const _tstring &message)
{
   OutputEx(_T("OnThreadPoolError - ") + message);
}

void CIOPool::OnThreadPoolThreadCreated()
{
   if (m_log)
   {
      Output(_T("OnThreadPoolThreadCreated"));
   }
}

void CIOPool::OnThreadPoolThreadStarted()
{
   if (m_log)
   {
      Output(_T("OnThreadPoolThreadStarted"));
   }
}

void CIOPool::OnThreadPoolDispatch()
{
   if (m_log)
   {
      Output(_T("OnThreadPoolDispatch"));
   }
}

void CIOPool::OnThreadPoolWorkItemsDequeued(
   const DWORD numWorkItems)
{
   if (m_log)
   {
      Output(_T("OnThreadPoolWorkItemsDequeued: ") + ToString(numWorkItems));
   }
}

void CIOPool::OnThreadPoolProcessingStarted(
   const bool dataAvailable)
{
   if (m_log)
   {
      Output(_T("OnThreadPoolProcessingStarted: ") + BoolAsString(dataAvailable));
   }
}

void CIOPool::OnThreadPoolProcessingStopped(
   const bool dataAvailable)
{
   if (m_log)
   {
      Output(_T("OnThreadPoolProcessingStopped: ") + BoolAsString(dataAvailable));
   }
}

void CIOPool::OnThreadPoolThreadStopped()
{
   if (m_log)
   {
      Output(_T("OnThreadPoolThreadStopped"));
   }
}

void CIOPool::OnThreadPoolThreadDestroyed()
{
   if (m_log)
   {
      Output(_T("OnThreadPoolThreadDestroyed"));
   }
}

///////////////////////////////////////////////////////////////////////////////
// End of file: IOPool.cpp
///////////////////////////////////////////////////////////////////////////////
