#pragma once
///////////////////////////////////////////////////////////////////////////////
// File: IOPool.h
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2004 JetByte Limited.
//
// This software is provided "as is" without a warranty of any kind. All
// express or implied conditions, representations and warranties, including
// any implied warranty of merchantability, fitness for a particular purpose
// or non-infringement, are hereby excluded. JetByte Limited and its licensors
// shall not be liable for any damages suffered by licensee as a result of
// using the software. In no event will JetByte Limited be liable for any
// lost revenue, profit or data, or for direct, indirect, special,
// consequential, incidental or punitive damages, however caused and regardless
// of the theory of liability, arising out of the use of or inability to use
// software, even if JetByte Limited has been advised of the possibility of
// such damages.
//
///////////////////////////////////////////////////////////////////////////////

#include "JetByteTools/IOTools/IOPool.h"
#include "JetByteTools/CoreTools/NullThreadPoolMonitor.h"

#include "JetByteTools/CoreTools/tstring.h"

///////////////////////////////////////////////////////////////////////////////
// CIOPool
///////////////////////////////////////////////////////////////////////////////

class CIOPool:
   public JetByteTools::IO::CIOPool,
   private JetByteTools::Core::CNullThreadPoolMonitor
{
   public :

      explicit CIOPool(
         ThreadCount numThreads = 0,
         bool log = true);

      CIOPool(
         const CIOPool &rhs) = delete;

      ~CIOPool() override;

      CIOPool &operator=(
         const CIOPool &rhs) = delete;

   private :

      void OnThreadPoolError(
         const JetByteTools::Core::_tstring &message) override;

      void OnThreadPoolThreadCreated() override;
      void OnThreadPoolThreadStarted() override;
      void OnThreadPoolDispatch() override;
      void OnThreadPoolWorkItemsDequeued(
         DWORD numWorkItems) override;
      void OnThreadPoolProcessingStarted(
         bool dataAvailable) override;
      void OnThreadPoolProcessingStopped(
         bool dataAvailable) override;
      void OnThreadPoolThreadStopped() override;
      void OnThreadPoolThreadDestroyed() override;

      const bool m_log;
};

///////////////////////////////////////////////////////////////////////////////
// End of file: IOPool.h
///////////////////////////////////////////////////////////////////////////////
